==> snowpack-2.4.0.tgz: package/dist-src/commands/paint.js <==
import chalk from 'chalk';
import readline from 'readline';
import util from 'util';
import { isYarn } from '../util';
const cwd = process.cwd();
function getStateString(workerState, isWatch) {
    if (workerState.state) {
        if (Array.isArray(workerState.state)) {
            return [chalk[workerState.state[1]], workerState.state[0]];
        }
        return [chalk.dim, workerState.state];
    }
    if (workerState.done) {
        return workerState.error ? [chalk.red, 'FAIL'] : [chalk.green, 'DONE'];
    }
    if (isWatch) {
        if (workerState.config.watch) {
            return [chalk.dim, 'WATCH'];
        }
    }
    return [chalk.dim, 'READY'];
}
const WORKER_BASE_STATE = { done: false, error: null, output: '' };
export function paint(bus, registeredWorkers, buildMode, devMode) {
    let consoleOutput = '';
    let installOutput = '';
    let isInstalling = false;
    let hasBeenCleared = false;
    let missingWebModule = null;
    const allWorkerStates = {};
    for (const config of registeredWorkers) {
        allWorkerStates[config.id] = { ...WORKER_BASE_STATE, config };
    }
    function repaint() {
        process.stdout.write(process.platform === 'win32' ? '\x1B[2J\x1B[0f' : '\x1B[2J\x1B[3J\x1B[H');
        process.stdout.write(`${chalk.bold('Snowpack')}\n\n`);
        // Dashboard
        if (devMode) {
            process.stdout.write(`  ${chalk.bold.cyan(`${devMode.protocol}//localhost:${devMode.port}`)}`);
            for (const ip of devMode.ips) {
                process.stdout.write(`${chalk.cyan(` > `)}${chalk.bold.cyan(`${devMode.protocol}//${ip}:${devMode.port}`)}`);
            }
            process.stdout.write('\n' + chalk.dim(`  Server started in ${devMode.startTimeMs}ms.\n\n`));
        }
        if (buildMode) {
            process.stdout.write('  ' + chalk.bold.cyan(buildMode.dest));
            process.stdout.write(chalk.dim(` Building your application...\n\n`));
        }
        for (const config of registeredWorkers) {
            if (devMode && config.type === 'bundle') {
                continue;
            }
            const workerState = allWorkerStates[config.id];
            const dotLength = 24 - config.id.length;
            const dots = chalk.dim(''.padEnd(dotLength, '.'));
            const [fmt, stateString] = getStateString(workerState, !!devMode);
            const spacer = ' '; //.padEnd(8 - stateString.length);
            let cmdMsg = `${config.plugin && config.cmd[0] !== '(' ? '(plugin) ' : ''}${config.cmd}`;
            if (cmdMsg.length > 52) {
                cmdMsg = cmdMsg.substr(0, 52) + '...';
            }
            const cmdStr = stateString === 'FAIL' ? chalk.red(cmdMsg) : chalk.dim(cmdMsg);
            process.stdout.write(`  ${config.id}${dots}[${fmt(stateString)}]${spacer}${cmdStr}\n`);
        }
        process.stdout.write('\n');
        if (isInstalling) {
            process.stdout.write(`${chalk.underline.bold('▼ snowpack install')}\n\n`);
            process.stdout.write('  ' + installOutput.trim().replace(/\n/gm, '\n  '));
            process.stdout.write('\n\n');
            return;
        }
        if (missingWebModule) {
            const { id, pkgName, spec } = missingWebModule;
            process.stdout.write(`${chalk.red.underline.bold('▼ Snowpack')}\n\n`);
            if (devMode) {
                process.stdout.write(`  Package ${chalk.bold(pkgName)} not found!\n`);
                process.stdout.write(chalk.dim(`  in ${id}`));
                process.stdout.write(`\n\n`);
                process.stdout.write(`  ${chalk.bold('Press Enter')} to automatically run ${chalk.bold(isYarn(cwd) ? `yarn add ${pkgName}` : `npm install --save ${pkgName}`)}.\n`);
                process.stdout.write(`  Or, Exit Snowpack and install manually to continue.\n`);
            }
            else {
                process.stdout.write(`  Dependency ${chalk.bold(spec)} not found!\n\n`);
                // process.stdout.write(
                //   `  Run ${chalk.bold('snowpack install')} to install all required dependencies.\n\n`,
                // );
                process.exit(1);
            }
            return;
        }
        for (const config of registeredWorkers) {
            const workerState = allWorkerStates[config.id];
            if (workerState && workerState.output) {
                const chalkFn = Array.isArray(workerState.error) ? chalk.red : chalk;
                process.stdout.write(`${chalkFn.underline.bold('▼ ' + config.id)}\n\n`);
                process.stdout.write(workerState.output
                    ? '  ' + workerState.output.trim().replace(/\n/gm, '\n  ')
                    : hasBeenCleared
                        ? chalk.dim('  Output cleared.')
                        : chalk.dim('  No output, yet.'));
                process.stdout.write('\n\n');
            }
        }
        if (consoleOutput) {
            process.stdout.write(`${chalk.underline.bold('▼ Console')}\n\n`);
            process.stdout.write(consoleOutput
                ? '  ' + consoleOutput.trim().replace(/\n/gm, '\n  ')
                : hasBeenCleared
                    ? chalk.dim('  Output cleared.')
                    : chalk.dim('  No output, yet.'));
            process.stdout.write('\n\n');
        }
        const overallStatus = Object.values(allWorkerStates).reduce((result, { done, error }) => {
            return {
                done: result.done && done,
                error: result.error || error,
            };
        });
        if (overallStatus.error) {
            process.stdout.write(`${chalk.underline.red.bold('▼ Result')}\n\n`);
            process.stdout.write('  ⚠️  Finished, with errors.');
            process.stdout.write('\n\n');
            process.exit(1);
        }
        else if (overallStatus.done) {
            process.stdout.write(`${chalk.underline.green.bold('▶ Build Complete!')}\n\n`);
        }
    }
    bus.on('WORKER_MSG', ({ id, msg }) => {
        allWorkerStates[id].output += msg;
        repaint();
    });
    bus.on('WORKER_UPDATE', ({ id, state }) => {
        if (typeof state !== undefined) {
            allWorkerStates[id].state = state;
        }
        repaint();
    });
    bus.on('WORKER_COMPLETE', ({ id, error }) => {
        allWorkerStates[id].state = null;
        allWorkerStates[id].done = true;
        allWorkerStates[id].error = allWorkerStates[id].error || error;
        repaint();
    });
    bus.on('WORKER_RESET', ({ id }) => {
        allWorkerStates[id] = { ...WORKER_BASE_STATE, config: allWorkerStates[id].config };
        repaint();
    });
    bus.on('CONSOLE', ({ level, args }) => {
        if (isInstalling) {
            const msg = util.format.apply(util, args);
            if (!msg.startsWith('[404] ')) {
                installOutput += msg;
            }
        }
        else {
            consoleOutput += `[${level}] ${util.format.apply(util, args)}\n`;
        }
        repaint();
    });
    bus.on('NEW_SESSION', () => {
        if (consoleOutput) {
            consoleOutput = ``;
            hasBeenCleared = true;
        }
        // Reset all per-file build scripts
        for (const config of registeredWorkers) {
            if (config.type === 'build') {
                allWorkerStates[config.id] = {
                    ...WORKER_BASE_STATE,
                    config: allWorkerStates[config.id].config,
                };
            }
        }
        repaint();
    });
    bus.on('INSTALLING', () => {
        isInstalling = true;
        installOutput = '';
        repaint();
    });
    bus.on('INSTALL_COMPLETE', () => {
        setTimeout(() => {
            missingWebModule = null;
            isInstalling = false;
            installOutput = '';
            consoleOutput = ``;
            hasBeenCleared = true;
            repaint();
        }, 2000);
    });
    bus.on('MISSING_WEB_MODULE', ({ id, data }) => {
        if (!missingWebModule && data) {
            missingWebModule = { id, ...data };
        }
        if (missingWebModule && missingWebModule.id === id) {
            if (!data) {
                missingWebModule = null;
            }
            else {
                missingWebModule = { id, ...data };
            }
        }
        repaint();
    });
    if (devMode) {
        readline.emitKeypressEvents(process.stdin);
        process.stdin.on('keypress', (str, key) => {
            if (key.name !== 'return' && key.name !== 'enter') {
                return;
            }
            if (!missingWebModule) {
                return;
            }
            devMode.addPackage(missingWebModule.pkgName);
            repaint();
        });
    }
    repaint();
}

==> snowpack-2.5.1.tgz: package/dist-src/commands/paint.js <==
import chalk from 'chalk';
import readline from 'readline';
import util from 'util';
import { isYarn } from '../util';
import detectPort from 'detect-port';
const cwd = process.cwd();
/**
 * Get the actual port, based on the `defaultPort`.
 * If the default port was not available, then we'll prompt the user if its okay
 * to use the next available port.
 */
export async function getPort(defaultPort) {
    const bestAvailablePort = await detectPort(defaultPort);
    if (defaultPort !== bestAvailablePort) {
        let useNextPort = false;
        if (process.stdout.isTTY) {
            const rl = readline.createInterface({ input: process.stdin, output: process.stdout });
            useNextPort = await new Promise((resolve) => {
                rl.question(chalk.yellow(`! Port ${chalk.bold(defaultPort)} not available. Run on port ${chalk.bold(bestAvailablePort)} instead? (Y/n) `), (answer) => {
                    resolve(!/^no?$/i.test(answer));
                });
            });
            rl.close();
        }
        if (!useNextPort) {
            console.error(chalk.red(`✘ Port ${chalk.bold(defaultPort)} not available. Use ${chalk.bold('--port')} to specify a different port.`));
            console.error();
            process.exit(1);
        }
    }
    return bestAvailablePort;
}
function getStateString(workerState, isWatch) {
    if (workerState.state) {
        if (Array.isArray(workerState.state)) {
            return [chalk[workerState.state[1]], workerState.state[0]];
        }
        return [chalk.dim, workerState.state];
    }
    if (workerState.done) {
        return workerState.error ? [chalk.red, 'FAIL'] : [chalk.green, 'DONE'];
    }
    if (isWatch) {
        if (workerState.config.watch) {
            return [chalk.dim, 'WATCH'];
        }
    }
    return [chalk.dim, 'READY'];
}
const WORKER_BASE_STATE = { done: false, error: null, output: '' };
export function paint(bus, registeredWorkers, buildMode, devMode) {
    let port;
    let protocol = '';
    let startTimeMs;
    let ips = [];
    let consoleOutput = '';
    let installOutput = '';
    let isInstalling = false;
    let hasBeenCleared = false;
    let missingWebModule = null;
    const allWorkerStates = {};
    for (const config of registeredWorkers) {
        allWorkerStates[config.id] = { ...WORKER_BASE_STATE, config };
    }
    function repaint() {
        process.stdout.write(process.platform === 'win32' ? '\x1B[2J\x1B[0f' : '\x1B[2J\x1B[3J\x1B[H');
        process.stdout.write(`${chalk.bold('Snowpack')}\n\n`);
        // Dashboard
        if (devMode) {
            const isServerStarted = startTimeMs > 0 && port > 0 && protocol;
            if (isServerStarted) {
                process.stdout.write(`  ${chalk.bold.cyan(`${protocol}//localhost:${port}`)}`);
                for (const ip of ips) {
                    process.stdout.write(`${chalk.cyan(` > `)}${chalk.bold.cyan(`${protocol}//${ip}:${port}`)}`);
                }
                process.stdout.write('\n');
                process.stdout.write(chalk.dim(startTimeMs < 1000 ? `  Server started in ${startTimeMs}ms.` : `  Server started.`));
                process.stdout.write('\n\n');
            }
            else {
                process.stdout.write(chalk.dim(`  Server starting…`) + '\n\n');
            }
        }
        if (buildMode) {
            process.stdout.write('  ' + chalk.bold.cyan(buildMode.dest));
            process.stdout.write(chalk.dim(` Building your application...\n\n`));
        }
        for (const config of registeredWorkers) {
            if (devMode && config.type === 'bundle') {
                continue;
            }
            const workerState = allWorkerStates[config.id];
            const dotLength = 24 - config.id.length;
            const dots = chalk.dim(''.padEnd(dotLength, '.'));
            const [fmt, stateString] = getStateString(workerState, !!devMode);
            const spacer = ' '; //.padEnd(8 - stateString.length);
            let cmdMsg = `${config.plugin && config.cmd[0] !== '(' ? '(plugin) ' : ''}${config.cmd}`;
            if (cmdMsg.length > 52) {
                cmdMsg = cmdMsg.substr(0, 52) + '...';
            }
            const cmdStr = stateString === 'FAIL' ? chalk.red(cmdMsg) : chalk.dim(cmdMsg);
            process.stdout.write(`  ${config.id}${dots}[${fmt(stateString)}]${spacer}${cmdStr}\n`);
        }
        process.stdout.write('\n');
        if (isInstalling) {
            process.stdout.write(`${chalk.underline.bold('▼ snowpack install')}\n\n`);
            process.stdout.write('  ' + installOutput.trim().replace(/\n/gm, '\n  '));
            process.stdout.write('\n\n');
            return;
        }
        if (missingWebModule) {
            const { id, pkgName, spec } = missingWebModule;
            process.stdout.write(`${chalk.red.underline.bold('▼ Snowpack')}\n\n`);
            if (devMode) {
                process.stdout.write(`  Package ${chalk.bold(pkgName)} not found!\n`);
                process.stdout.write(chalk.dim(`  in ${id}`));
                process.stdout.write(`\n\n`);
                process.stdout.write(`  ${chalk.bold('Press Enter')} to automatically run ${chalk.bold(isYarn(cwd) ? `yarn add ${pkgName}` : `npm install --save ${pkgName}`)}.\n`);
                process.stdout.write(`  Or, Exit Snowpack and install manually to continue.\n`);
            }
            else {
                process.stdout.write(`  Dependency ${chalk.bold(spec)} not found!\n\n`);
                // process.stdout.write(
                //   `  Run ${chalk.bold('snowpack install')} to install all required dependencies.\n\n`,
                // );
                process.exit(1);
            }
            return;
        }
        for (const config of registeredWorkers) {
            const workerState = allWorkerStates[config.id];
            if (workerState && workerState.output) {
                const chalkFn = Array.isArray(workerState.error) ? chalk.red : chalk;
                process.stdout.write(`${chalkFn.underline.bold('▼ ' + config.id)}\n\n`);
                process.stdout.write(workerState.output
                    ? '  ' + workerState.output.trim().replace(/\n/gm, '\n  ')
                    : hasBeenCleared
                        ? chalk.dim('  Output cleared.')
                        : chalk.dim('  No output, yet.'));
                process.stdout.write('\n\n');
            }
        }
        if (consoleOutput) {
            process.stdout.write(`${chalk.underline.bold('▼ Console')}\n\n`);
            process.stdout.write(consoleOutput
                ? '  ' + consoleOutput.trim().replace(/\n/gm, '\n  ')
                : hasBeenCleared
                    ? chalk.dim('  Output cleared.')
                    : chalk.dim('  No output, yet.'));
            process.stdout.write('\n\n');
        }
        const overallStatus = Object.values(allWorkerStates).reduce((result, { done, error }) => {
            return {
                done: result.done && done,
                error: result.error || error,
            };
        });
        if (overallStatus.error) {
            process.stdout.write(`${chalk.underline.red.bold('▼ Result')}\n\n`);
            process.stdout.write('  ⚠️  Finished, with errors.');
            process.stdout.write('\n\n');
            process.exit(1);
        }
        else if (overallStatus.done) {
            process.stdout.write(`${chalk.underline.green.bold('▶ Build Complete!')}\n\n`);
        }
    }
    bus.on('WORKER_MSG', ({ id, msg }) => {
        allWorkerStates[id].output += msg;
        repaint();
    });
    bus.on('WORKER_UPDATE', ({ id, state }) => {
        if (typeof state !== undefined) {
            allWorkerStates[id].state = state;
        }
        repaint();
    });
    bus.on('WORKER_COMPLETE', ({ id, error }) => {
        allWorkerStates[id].state = null;
        allWorkerStates[id].done = true;
        allWorkerStates[id].error = allWorkerStates[id].error || error;
        repaint();
    });
    bus.on('WORKER_RESET', ({ id }) => {
        allWorkerStates[id] = { ...WORKER_BASE_STATE, config: allWorkerStates[id].config };
        repaint();
    });
    bus.on('CONSOLE', ({ level, args }) => {
        if (isInstalling) {
            const msg = util.format.apply(util, args);
            if (!msg.startsWith('[404] ')) {
                installOutput += msg;
            }
        }
        else {
            consoleOutput += `[${level}] ${util.format.apply(util, args)}\n`;
        }
        repaint();
    });
    bus.on('NEW_SESSION', () => {
        if (consoleOutput) {
            consoleOutput = ``;
            hasBeenCleared = true;
        }
        // Reset all per-file build scripts
        for (const config of registeredWorkers) {
            if (config.type === 'build') {
                allWorkerStates[config.id] = {
                    ...WORKER_BASE_STATE,
                    config: allWorkerStates[config.id].config,
                };
            }
        }
        repaint();
    });
    bus.on('INSTALLING', () => {
        isInstalling = true;
        installOutput = '';
        repaint();
    });
    bus.on('INSTALL_COMPLETE', () => {
        setTimeout(() => {
            missingWebModule = null;
            isInstalling = false;
            installOutput = '';
            consoleOutput = ``;
            hasBeenCleared = true;
            repaint();
        }, 2000);
    });
    bus.on('MISSING_WEB_MODULE', ({ id, data }) => {
        if (!missingWebModule && data) {
            missingWebModule = { id, ...data };
        }
        if (missingWebModule && missingWebModule.id === id) {
            if (!data) {
                missingWebModule = null;
            }
            else {
                missingWebModule = { id, ...data };
            }
        }
        repaint();
    });
    bus.on('SERVER_START', (info) => {
        startTimeMs = info.startTimeMs;
        port = info.port;
        protocol = info.protocol;
        ips = info.ips;
        repaint();
    });
    if (devMode) {
        readline.emitKeypressEvents(process.stdin);
        process.stdin.on('keypress', (str, key) => {
            if (key.name !== 'return' && key.name !== 'enter') {
                return;
            }
            if (!missingWebModule) {
                return;
            }
            devMode.addPackage(missingWebModule.pkgName);
            repaint();
        });
    }
    repaint();
}
